// Read the data of SRF08/SRF10 Ultrasonic Range Finder
//

#include <stdio.h>
#include "roboard.h"
#define SRF_ADDR  (0xe0 >> 1)  // set SRF08/SRF10 address 0xe0

#ifdef WINCE
	#include <windows.h>
#endif

#ifdef WINCE
int _tmain(int argc, _TCHAR* _argv[])
{
	char  argvbuf[50][100] = {'\0'};
	char* argv[50];

	if (argc > 50) argc = 50;
	for (int _argc = 0; _argc < argc; _argc++)
	{
		argv[_argc] = &(argvbuf[_argc][0]);
		wcstombs(argv[_argc], _argv[_argc], 100);
	}
#else
int main(int argc, char* argv[])
{
#endif

	int c, val, max_range = 255;
	unsigned char buf[2] = {0}, data[2] = {0};
	void wait_ms(long t);

	// set the correct RoBoard version
	roboio_SetRBVer(RB_100);
	//roboio_SetRBVer(RB_100RD); // if your RoBoard is RB-100RD
	//roboio_SetRBVer(RB_110);   // if your RoBoard is RB-110

	// initialize I2C to set fast mode and 400K bps
	if(i2c_Init(I2CMODE_FAST, 400000UL) == false) {
		printf("ERROR: fail to init I2C lib (%s)!\n", roboio_GetErrMsg());
		return -1;
	}

	// get a maximum range from command-line argument
	if(argc > 1)
		max_range = atoi(argv[1]);

	// change the maximum range of SRF08/SRF10
	if(max_range <= 255) {
		buf[0] = 0x02;                      // set SRF range register (location 2)
		buf[1] = (unsigned char)max_range;  // set the maximum range
		i2c_Send(SRF_ADDR, buf, 2);
	}

	while(1) {
		printf("Press ENTER to read SRF08/SRF10 Ultrasonic Range Finder. ('Q' to exit.) ");
		if((c = getchar()) == 'Q' || c == 'q')
			break;

		buf[0] = 0x00;     // set SRF command register (location 0)
		buf[1] = 0x51;     // set ranging Mode - result in cm
		i2c_Send(SRF_ADDR, buf, 2);

		wait_ms(100);

		// read SRF data from location 2
		i2c_SensorRead(SRF_ADDR, 0x02, data, 2);

		val = ((int)data[0] << 8) | data[1];
		printf("distance = %4d cm\n\n", val);
	}

	i2c_Close();

	return 0;
}

void wait_ms(long t)
{
    // This's a function for t-millisecond delay; implement this according to your platform!
    // The following is just a stupid (but simple) cross-platform implementation for the purpose of sample codes.

    volatile long i, j;
    for (i=0L; i<t; i++)
    for (j=0L; j<66666L; j++);
}
